import unittest

import numpy as np

from conf.testing import czifile, ilastik, output_path
from model_server.image import CziImageFileAccessor, InMemoryDataAccessor, write_accessor_data_to_file
from model_server.ilastik import IlastikPixelClassifierModel

class TestIlastikPixelClassification(unittest.TestCase):
    def setUp(self) -> None:
        self.cf = CziImageFileAccessor(czifile['path'])

    # def tearDown(self) -> None:
    #     pass

    def test_faulthandler(self): # recreate error that is messing up ilastik
        import io
        import sys
        import faulthandler

        with self.assertRaises(io.UnsupportedOperation):
            faulthandler.enable(file=sys.stdout)

    def test_run_pixel_classifier_on_random_data(self):
        model = IlastikPixelClassifierModel({'project_file': ilastik['pixel_classifier']})
        w = 512
        h = 256

        input_img = InMemoryDataAccessor(data=np.random.rand(w, h, 1, 1))

        pxmap = model.infer(input_img)
        self.assertEqual(pxmap.shape, (w, h, 2, 1))

    def test_run_pixel_classifier(self):
        channel = 2
        model = IlastikPixelClassifierModel({'project_file': ilastik['pixel_classifier']})
        cf = CziImageFileAccessor(czifile['path'])
        mono_image = cf.get_one_channel_data(channel)

        self.assertEqual(mono_image.shape_dict['X'], czifile['w'])
        self.assertEqual(mono_image.shape_dict['Y'], czifile['h'])
        self.assertEqual(mono_image.shape_dict['C'], 1)
        self.assertEqual(mono_image.shape_dict['Z'], 1)

        pxmap = model.infer(mono_image)

        self.assertEqual(pxmap.shape[0:2], cf.shape[0:2])
        self.assertEqual(pxmap.shape_dict['C'], 2)
        self.assertEqual(pxmap.shape_dict['Z'], 1)
        print(pxmap.shape_dict)
        self.assertTrue(
            write_accessor_data_to_file(
                output_path / f'pxmap_{cf.fpath.stem}_ch{channel}.tif',
                pxmap
            )
        )

